/* This driver was initially made in my project report.
 * It has been customized for this use.
 *
 * Sindre Hansen (2011).
 */


#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <asm/io.h>
#include <asm/uaccess.h>

#define FPGA_BASE 0x81000000
#define FPGA_MAX 0x810000ff


MODULE_LICENSE("GPL");

static int fpga_major = 240;
static void *io_base;

static int fpga_init(void);
static void fpga_exit(void);
static ssize_t fpga_read(struct file *filp, char *buf, size_t count, loff_t *f_pos);
static ssize_t fpga_write(struct file *filp, unsigned char *buf, size_t count, loff_t *f_pos);

struct file_operations fpga_fops = {
	read:     fpga_read,
	write:    fpga_write,
	poll:     NULL,
	open:     NULL,
	release:  NULL,
};


ssize_t fpga_read(struct file *filp, char *buf, size_t count, loff_t *f_pos)
{
	int retval;
	void *add;
	unsigned char *kbuf, *ptr;

	kbuf = kmalloc(count, GFP_KERNEL);
	if (!kbuf) return -ENOMEM;
	ptr = kbuf;
	retval = count;

	int i;
	for(i = *f_pos; i < count + *f_pos; i++)
	{
		add = io_base + i;
		ptr[i - *f_pos] = readb(add);		
	}

	copy_to_user(buf, kbuf, retval);


	kfree(kbuf);
	*f_pos += retval;	

	return retval;
}

ssize_t fpga_write(struct file *filp, unsigned char *buf, size_t count, loff_t *f_pos)
{
	unsigned char *kbuf, *ptr;
	kbuf = kmalloc(count, GFP_KERNEL);

	int ret;
        ret = copy_from_user(kbuf, buf, count);

	void* location = ioremap(FPGA_BASE, (long)count);
	int i;

	for(i = *f_pos; i < count + *f_pos; i++)
	{
		writeb(kbuf[i - *f_pos], location + i);
	}

	iounmap(location);
	kfree(kbuf);

	return 0;
}



static int fpga_init(void)
{
	int result = register_chrdev(fpga_major, "/var/swreg", &fpga_fops);

	io_base = ioremap(FPGA_BASE, FPGA_MAX - FPGA_BASE);
	printk("Major number captured: %d\n",result);
	return 0;
}

static void fpga_exit(void)
{
        iounmap(io_base);
	unregister_chrdev(fpga_major, "/var/swreg");
	printk("Closing register driver.\n");
}

module_init(fpga_init);
module_exit(fpga_exit);


