#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <hicap.h>
#include "test_dff.h"

static void reconfigure_fpga_(char* bitfile);
static const char* icapdevice = "/var/icap";
static const int icapframes = 66;
static int icaphandle = -1;


static void reconfigure_fpga_(char* bitfile)
{
        hicap_write(icaphandle, bitfile, 0, 21, 0, icapframes);
}



int main()
{
        printf("Self-reconfiguration of synchronous modules on FPGA\n");
        printf("Prerequisites:\n");
        printf("Development board: Suzaku-V.\n");
        printf("Processor: Power-PC.\n");
        printf("FPGA: Virtex-4 XC4VFX12.\n");
        printf("OS: atmark-dist-20090318 (linux-2.6.18-at11).\n");
        printf("Base FPGA-design: bitfiles/dff_with_bufgce_with_dirt.bit (must be uploaded with netflash).\n\n");
        printf("These tests will perform partial reconfiguration of synchronous modules on the FPGA.\n");
        printf("For each time the FPGA is reconfigured, the new design will be thoroughly tested.\n");

        //! Set up device drivers.
        system("mknod /var/swreg c 240 0");
        system("insmod ../drivers/sw_access_reg.ko");
        system("mknod /var/icap c 250 0");
        system("insmod ../drivers/xilinx_hwicap_m.ko");

        int number_failed = 0;

        icaphandle = hicap_open((char*)icapdevice, "XC4VFX12");

        printf("===\n");
        printf("Inserting module: One flip-flop.\n");
        reconfigure_fpga_("../bitfiles/onedff_ise_part.bit");
        printf("===\n");
        number_failed += test_onedff();
        printf("===\n");
        printf("Inserting module: Three flip-flops.\n");
        reconfigure_fpga_("../bitfiles/threedffs_ise_part.bit");
        printf("===\n");
        number_failed += test_threedffs();
        printf("===\n");
        printf("Inserting module: One flip-flop (shifted one column left).\n");
        reconfigure_fpga_("../bitfiles/onedff_ise_shifted_left_part.bit");
        printf("===\n");
        number_failed += test_onedff();
        printf("===\n");
        printf("Inserting module: Three flip-flops (shifted one column left).\n");
        reconfigure_fpga_("../bitfiles/threedffs_ise_shifted_left_part.bit");
        printf("===\n");
        number_failed += test_threedffs();
        hicap_close(icaphandle);

        printf("\nSummary:\n");
        if (number_failed == 0)
                printf("All tests passed!\n");
        else
                printf("Test suites that failed: %d\n.", number_failed);

        system("rmmod sw_access_reg");
        system("rmmod xilinx_hwicap_m");
        system("rm /var/swreg");
        system("rm /var/icap");

        return 0;
}


