#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <signal.h>
//! Message queue functions.
#include <sys/ipc.h> 
#include <sys/msg.h>
//! Sleep, unused at the moment.
#include <unistd.h>
#include <pthread.h>

#include <hlog.h>
#include <hmsg.h>
#include <hmqueue.h>
#include <hmem.h>
// Not used.
//#include <devicelist.h>
#include <integer_functions.h>
#include <hstructures.h>
#include <hprocess.h>
#include <hevent.h>

#include "hdaemon.h"
#include "hdsched.h"
#include "hdmsg.h"
#include "hdplacer.h"
#include "hdtimer.h"

/*! \brief Process a received message.
 *
 *      @param req_msg The message to be processed.
 *      @return The message to be sent back to client on success. NULL on failure.
 */
static void* process_request_(void* req_msg);
static void cleanup_(int exit_code);
static int handle_event_();

//! The event handler for this thread.
static struct hevent* event_handler = NULL;


static int handle_event_()
{
        enum hdmevent incoming_event = (enum hdmevent)hevent_wait(event_handler);

        // An incoming event is handled here.
        switch (incoming_event) {
                case HDME_QUIT:
                        // Quit the thread.
                        printf("wait_event_: Exiting message server thread.\n");
                        cleanup_(EXIT_SUCCESS);
                        break;
        }

        return 0;
}


void* process_request_(void* req_msg)
{
        hlog_write(HLOG_NORMAL, "process_request_: Processing message.\n");

        // The response message to be returned.
        void* resp_msg = NULL;

        hlog_write(HLOG_DEBUG, "process_request_: command=");
        hlog_write_integer(hmsg_get_command(req_msg));
        hlog_write_text(".\n");

        switch (hmsg_get_command(req_msg)) {
                // GCC compiler demands brackets on this case. 
                case HMC_ALLOC:	{
                                        // Commented away. Not ready to be used.
                                                /*
                                                hlog_write(HLOG_NORMAL, "process_request_: HMC_ALLOC\n");
                                                resp_msg = hmsg_create(HMT_CTRL);
                                                int* base_addr = calloc(1, sizeof(*base_addr));
                                                int alloc_ret = hmem_allocate(char_to_int(hmsg_get_data(req_msg), 0), base_addr);
                                                if (alloc_ret < 0) {
                                                        hlog_write(HLOG_ERROR, "process_request_: Failed to allocate memory for HMC_ALLOC.\n");
                                                } else {
                                                        hlog_write(HLOG_DEBUG, "process_request_: Allocated memory for HMC_ALLOC.\n");
                                                        hmsg_set_return(resp_msg, *base_addr);
                                                }
                                                free(base_addr);
                                                base_addr = NULL;
                                                alloc_ret = -1;
                                                */
                                                break;
                                        }
                case HMC_EXEC:
                                        hlog_write(HLOG_NORMAL, "process_request_: HMC_EXEC\n");
                                        resp_msg = hmsg_create(HMT_CTRL);
                                        // Not ready yet.
                                        //hmem_instruction_write(0, hmsg_get_data(req_msg), hmsg_get_size(req_msg));
                                        hmsg_set_return(resp_msg, HMR_OK);
                                        break;		
                case HMC_FREE:
                                        hlog_write(HLOG_NORMAL, "process_request_: HMC_FREE\n");
                                        resp_msg = hmsg_create(HMT_CTRL);
                                        // Not ready yet.
                                        //hmem_free(char_to_int(hmsg_get_data(req_msg), 0));
                                        hmsg_set_return(resp_msg, HMR_OK);
                                        break;	
                case HMC_LDDEV:	
                                        hlog_write(HLOG_NORMAL, "process_request_: HMC_LDDEV\n");
                                        resp_msg = hmsg_create(HMT_CTRL);
                                        hmsg_set_return(resp_msg, HMR_OK);
                                        break;
                case HMC_RMDEV:
                                        hlog_write(HLOG_NORMAL, "process_request_: HMC_RMDEV\n");
                                        resp_msg = hmsg_create(HMT_CTRL);
                                        hmsg_set_return(resp_msg, HMR_OK);
                                        break;
                case HMC_RMQUE: {
                                         // Connect to an existing queue.
                                         // Note: If the queue is registered already, the old queue will be removed
                                         // and the new queue registered instead.
                                         hlog_write(HLOG_NORMAL, "process_request_: HMC_RMQUE\n");
                                         resp_msg = hmsg_create(HMT_CTRL);
                                         // Connect to an existing queue. Use sender's identifier as key.
                                         struct hmqueue* reg_mqueue = hmqueue_add(hmsg_get_sender(req_msg), HMQ_CONNECT);
                                         if (reg_mqueue == NULL) {
                                                 hlog_write(HLOG_ERROR, "process_request_: Could not connect to message queue.\n");
                                                 hmsg_set_return(resp_msg, HMR_ERROR);
                                         } else {
                                                 hlog_write(HLOG_DEBUG, "process_request_: Connected to message queue. msqid_req=");
                                                 hlog_write_integer(hmqueue_get_id(reg_mqueue));
                                                 hlog_write_text(". key=");
                                                 hlog_write_integer(hmqueue_get_key(reg_mqueue));
                                                 hlog_write_text(".\n");
                                                 hmsg_set_return(resp_msg, HMR_OK);
                                         }
                                         reg_mqueue = NULL;
                                         break;
                                 }
                case HMC_READ:
                                 hlog_write(HLOG_NORMAL, "process_request_: HMC_READ\n");
                                 resp_msg = hmsg_create(HMT_CTRLMEM);
                                 // Not ready yet.
                                 //hmem_data_read(0, hmsg_get_data(resp_msg), hmsg_get_address(req_msg), hmsg_get_size(req_msg));
                                 hmsg_set_return(resp_msg, HMR_OK);
                                 break;
                case HMC_UMQUE:
                                 hlog_write(HLOG_NORMAL, "process_request_: HMC_UMQUE\n");
                                 hmqueue_remove_by_key(hmsg_get_sender(req_msg));
                                 break;
                case HMC_WRITE:
                                 hlog_write(HLOG_NORMAL, "process_request_: HMC_WRITE\n");
                                 resp_msg = hmsg_create(HMT_CTRL);
                                 // Not ready yet.
                                 //hmem_data_write(0, hmsg_get_data(req_msg), hmsg_get_address(req_msg), hmsg_get_size(req_msg));
                                 hmsg_set_return(resp_msg, HMR_OK);
                                 break;
                case HMC_REGPROC:
                                 // Register a new process and wake up scheduler.
                                 hlog_write(HLOG_NORMAL, "process_request_: HMC_REGPROC\n");
                                 resp_msg = hmsg_create(HMT_CTRL);
                                 struct hprocess* process_new = hprocess_create(hmsg_get_nice(req_msg));
                                 hlog_write(HLOG_DEBUG, "process_request_: Filename in msg: ");
                                 hlog_write_text(hmsg_get_bitfilename(req_msg));
                                 hlog_write_text("\n");
                                 hprocess_set_bitfilename( process_new, hmsg_get_bitfilename(req_msg) );
                                 if (process_new == NULL) {
                                         hmsg_set_return(resp_msg, HMR_NOPID);
                                 } else {
                                         hprocess_set_bitfilename(process_new, hmsg_get_bitfilename(req_msg));
                                         hmsg_set_return(resp_msg, HMR_OK);
                                         hdsched_add_new_process(process_new);
                                         hdsched_notify(HDSE_NEW_PROCESS);
                                 }
                                 break;
                default:
                                 break;
        }

        return resp_msg;
}


static void cleanup_(int exit_code)
{	
        if (exit_code == EXIT_FAILURE) {
                hlog_write(HLOG_ERROR, "HWOS message server terminated with error=");
                hlog_write_integer(errno);
                hlog_write_text(".\n");
        }
        else
                hlog_write(HLOG_NORMAL, "HWOS message server terminated normally.\n");

        // Quit the other threads also.
        hdtimer_notify(HDTE_QUIT);
        hdplacer_notify(HDPE_QUIT);
        hdsched_notify(HDSE_QUIT);

        hevent_remove(event_handler);
        pthread_exit(NULL);
}


int hdmsg_notify(enum hdmevent event)
{
        hevent_notify(event_handler, (int)event);

        return 0;
}


int hdmsg_synchronize()
{
        // Send dummy/sync message to message server
        // (to make sure it handles event).
        void* sync_msg = hmsg_create(HMT_CTRL);
        hmsg_send(sync_msg, hmqueue_get(hmqueue_get_daemonkey()));
        free(sync_msg);

        return 0;
}


void* hdmsg_main()
{
        // Init the message queue and return the identifier.
        struct hmqueue* mqueue;
        mqueue = hmqueue_add(hmqueue_get_daemonkey(), HMQ_CREATE);

        // Message to be received.
        void* req_msg;

        // Create event queue.
        event_handler = hevent_create(HEVENT_NONBLOCKING);
        if (event_handler == NULL) {
                cleanup_(EXIT_FAILURE);
        }

        // Service loop for message server.
        while (1) {
                // Blocking wait for a message on queue msqid. Put message in req_msg.
                hlog_write(HLOG_NORMAL, "hdmsg_main: Waiting for message.\n");
                req_msg = hmsg_receive(mqueue);
                
                if (req_msg == NULL) {
                        hlog_write(HLOG_ERROR, "hdmsg_main: req_msg is NULL.\n");
                        cleanup_(EXIT_FAILURE);
                }

                void* resp_msg;
                // Process the received message and make ready the response message.
                resp_msg = process_request_(req_msg); 

                if (resp_msg != NULL) {
                        // Get the message queue for the sender and send the response. 
                        hmsg_send(resp_msg, hmqueue_get(hmsg_get_sender(req_msg)));
                }

                // Handle any events from the other threads.
                handle_event_();
        }

        cleanup_(EXIT_SUCCESS);
}






