#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <unistd.h>

#include <hevent.h>

#include "hdplacer.h"
#include "hdmsg.h"
#include "hdtimer.h"


static void cleanup_(int exit_code);

/*! \brief Handle an incoming event.
 *
 *  This functions locks a mutex, but does not wait on a
 *  condition variable (events are handled synchronously).
 */
static int handle_event_();

//! The event handler for this thread.
static struct hevent* event_handler = NULL;

//! If this is true, this thread should ask the server to quit all threads when exiting.
static int notify_server_quit = 1;


static int handle_event_()
{
        enum hdtevent incoming_event = (enum hdtevent)hevent_wait(event_handler); 

        // An incoming event is handled here.
        switch (incoming_event) {
                case HDTE_QUIT:
                        // Do not ask server to quit (the server asked this thread to quit).
                        notify_server_quit = 0;
                        // Quit the thread.
                        cleanup_(EXIT_SUCCESS);
                        break;
        }
        
        return 0;
}


static void cleanup_(int exit_code)
{	
        // Only notify server if not done already.
        if (notify_server_quit) {
                notify_server_quit = 0;
                hdmsg_notify(HDME_QUIT);
        }

        hdmsg_synchronize();
        pthread_exit(NULL);
}


int hdtimer_notify(enum hdtevent event)
{
        hevent_notify(event_handler, (int)event);

        return 0;
}

const int hdtimer_get_timeslice()
{
        return 1000000;
}


void* hdtimer_main()
{
        // Create event handler.
        event_handler = hevent_create(HEVENT_NONBLOCKING);
        if (event_handler == NULL) {
                notify_server_quit = 0;
                cleanup_(EXIT_FAILURE);
        }

        // Sleep a bit to initialize scheduler.
        sleep(1);
        // Service loop for placer.
        while (1) {
                // Wait for one timeslice.
                usleep(hdtimer_get_timeslice());
                // Notify placer that timer has expired.
                hdplacer_notify(HDPE_TIMER);
                // Handle an incoming event.
                handle_event_();
        }

        hevent_remove(event_handler);
        cleanup_(EXIT_SUCCESS);
}






