#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hdev.h"
#include "hstructures.h"

static struct hdev* firstd = NULL;
static struct hdev* lastd = NULL;
static int dev_cnt = 0;

// Device driver structure.
struct hdev {
	int major;
	char devicePath[HDEV_PATH_SIZE];
        struct hdev* next;
        struct hdev* prev;
};


static int set_major_(struct hdev* device, int major);
static int set_path_(struct hdev* device, char* path);
static int set_next_(struct hdev* device, struct hdev* next);
static int set_prev_(struct hdev* device, struct hdev* prev);
static int set_last_(struct hdev* last);
static int set_first_(struct hdev* first);
static int decrement_size_();
static int increment_size_();


static int increment_size_()
{
        dev_cnt += 1;

        return dev_cnt;
}


static int decrement_size_()
{
        dev_cnt -= 1;

        if (dev_cnt < 0)
                dev_cnt = 0;

        return dev_cnt;
}


static int set_last_(struct hdev* last)
{
        lastd = last;

        return 0;
}


static int set_first_(struct hdev* first)
{
        firstd = first;

        return 0;
}


static int set_next_(struct hdev* device, struct hdev* next)
{
        if (device == NULL)
                return -1;

        device->next = next;

        return 0;
}


static int set_prev_(struct hdev* device, struct hdev* prev)
{
        if (device == NULL)
                return -1;

        device->prev = prev;

        return 0;
}


static int set_major_(struct hdev* device, int major)
{
        if (device == NULL)
                return -1;

        device->major = major;

        return 0;
}


static int set_path_(struct hdev* device, char* path)
{
        if (device == NULL)
                return -1;

        if (strncpy(device->devicePath, path, HDEV_PATH_SIZE) != device->devicePath)
                return -1;

        return 0;
}


int hdev_size()
{
        return dev_cnt;
}


int hdev_get_major(struct hdev* device)
{
        if (device == NULL)
                return -1;

        return device->major; 
}


struct hdev* hdev_get_by_major(int major)
{
        if (hdev_size() == 0)
                return NULL;

        struct hdev* device = hdev_get_first();

        if (hdev_get_major(device) == major)
                return device;
        
        // TODO: What if get_next never returns NULL?
        while (hdev_get_next(device) != NULL) {
                device = hdev_get_next(device);
                if (hdev_get_major(device) == major)
                        return device;
        }

        // Nothing found => return NULL.
        return NULL;
}


struct hdev* hdev_add(int major, char* device_path)
{
        if (hdev_get_by_major(major) != NULL)
                return NULL;

        struct hdev* device = calloc(1, sizeof(struct hdev));

        set_major_(device, major);
        set_path_(device, device_path);

        if (hdev_size() == 0)
                set_first_(device);

        set_prev_(device, hdev_get_last());
        set_next_(device, NULL);
        set_next_(hdev_get_last(), device);	

        set_last_(device);

        increment_size_();
        
        return device;
}


int hdev_remove(struct hdev* device)
{
        if (device == NULL)
                return -1;

        set_prev_(hdev_get_next(device), hdev_get_prev(device));
        set_next_(hdev_get_prev(device), hdev_get_next(device));
        
        if (hdev_get_last() == device)
                set_last_(hdev_get_prev(device));

        if (hdev_get_first() == device)
                set_first_(hdev_get_next(device));

        free(device);
        decrement_size_();

        return 0;
}


int hdev_remove_by_major(int major)
{
        struct hdev* device = hdev_get_by_major(major);
        return hdev_remove(device);
}


int hdev_remove_all()
{
        struct hdev* device = hdev_get_first();

        int number_removed = 0;
        while (hdev_get_next(device) != NULL) {
                device = hdev_get_next(device);
                if (hdev_remove(hdev_get_prev(device)) == 0)
                        number_removed += 1;
        }

        if (hdev_remove(device) == 0)
                number_removed += 1;

        return number_removed;
}


struct hdev* hdev_get_first()
{
        return firstd; 
}


struct hdev* hdev_get_last()
{
        return lastd; 
}


struct hdev* hdev_get_next(struct hdev* device)
{
        if (device == NULL)
                return NULL;

        return device->next;
}


struct hdev* hdev_get_prev(struct hdev* device)
{
        if (device == NULL)
                return NULL;

        return device->prev;
}


