#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>

#include "hevent.h"


//! Event handler structure. Has everything to handle and keep track of events.
struct hevent {
        int msqid;                           //! Message queue ID.
        int blocking;                        //! True if wait should block until a new event.
};


struct hevent_msg {
            long mtype;
            int mtext;
};


struct hevent* hevent_create(int blocking)
{
        if (blocking > 1 || blocking < 0)
                return NULL;

        struct hevent* event = calloc(1, sizeof(struct hevent));
        event->msqid = msgget(IPC_PRIVATE, 0644 | IPC_CREAT | IPC_EXCL);
        event->blocking = blocking;

        if (event->msqid == -1)
                return NULL;

        return event;
}


int hevent_notify(struct hevent* event, int notification)
{
        if (event == NULL)
                return -1;

        struct hevent_msg msg;
        msg.mtype = 1;
        msg.mtext = notification;
        if (msgsnd(event->msqid, &msg, sizeof(msg.mtext), 0) == -1)
                return -1;

        return 0;
}


int hevent_wait(struct hevent* event)
{
        if (event == NULL)
                return -1;

        struct hevent_msg msg;

        int msgflg = IPC_NOWAIT;
        if (event->blocking)
                msgflg = 0;
        
        if (msgrcv(event->msqid, &msg, sizeof(msg.mtext), 0, msgflg) == -1)
                return -1;

        return msg.mtext;
}


int hevent_remove(struct hevent* event)
{
        if (event == NULL)
                return -1;

        if (msgctl(event->msqid, IPC_RMID, NULL) == -1)
                return -1;
        else
                return 0;
}


