#include <stdio.h>
#include <pthread.h>

#include "hlog.h"

static FILE *hlogfile = NULL;

//! Mutex for writing to file.
static pthread_mutex_t write_mutex = PTHREAD_MUTEX_INITIALIZER;

int hlog_init(char* logpath) {
	hlogfile = fopen(logpath,"w");
        if (hlogfile == NULL)
                return -1;
        else
                return 0;
}

void hlog_write(enum hlog_flag flag, char* text)
{
        if (hlogfile == NULL)
                return;
        
        
        pthread_mutex_lock(&write_mutex); 

        switch (flag) {
                case HLOG_ERROR:
                        fprintf(hlogfile, "ERROR: ");
                        break;
                case HLOG_WARNING:
                        fprintf(hlogfile, "WARNING: ");
                        break;
                case HLOG_NORMAL:
                        fprintf(hlogfile, "NORMAL: ");
                        break;
                case HLOG_DEBUG:
                        fprintf(hlogfile, "DEBUG: ");
                        break;
                default:
                        fprintf(hlogfile, "UNKNOWN: ");
                        break;
        }

        fprintf(hlogfile,"%s", text);
        fflush(hlogfile);

        pthread_mutex_unlock(&write_mutex);
}

void hlog_write_text(char* text)
{
        if (hlogfile == NULL)
                return;

        pthread_mutex_lock(&write_mutex); 

        fprintf(hlogfile,"%s", text);
        fflush(hlogfile);

        pthread_mutex_unlock(&write_mutex);
}

void hlog_write_integer(int number)
{
        if (hlogfile == NULL)
                return;

        pthread_mutex_lock(&write_mutex); 

        fprintf(hlogfile,"%d", number);
        fflush(hlogfile);

        pthread_mutex_unlock(&write_mutex);
}

void hlog_close()
{
        if (hlogfile == NULL)
                return;

	fclose(hlogfile);
}

