/*! \file hlog.h
 *  \brief [Hardware OS log interface] Log file functions for HWOS.
 *
 *  Functionality for writing to a log file.
 *  Writing to the log is mutual exclusive and thread safe.
 *
 *  Author (2011): Sindre Hansen
 *
 */

#ifndef HLOG_H
#define HLOG_H

//! Definition of possible flags for log entries.
enum hlog_flag {
        HLOG_ERROR,
        HLOG_WARNING,
        HLOG_NORMAL,
        HLOG_DEBUG
};

/*! \brief Open the log file for the HWOS. 
 *
 *  @return Positive value on success. Negative value on failure.
 */
int hlog_init();

/*! \brief Write a flag and a string to the log file. 
 *
 *  @param flag Flag to write. Could for example indicate an error or a warning.
 *  @param text Character array (string) to be written.
 */
void hlog_write(enum hlog_flag flag, char* text);

/*! \brief Write a string to the log file. 
 *
 *  @param text Character array (string) to be written.
 */
void hlog_write_text(char* text);


/*! \brief Write an integer to the log file. 
 *
 *  @param number Integer value to be written.
 */
void hlog_write_integer(int number);

/*! \brief Close the log file for the HWOS. 
 *
 */
void hlog_close();


#endif

