/*! \file hvmemptr.h
 *  \brief [Hardware OS Virtual Memory Pointer Interface] Functionality for virtual BRAM pointers.
 *
 *  Author (2011): Sindre Hansen
 */


#ifndef HVMEMPTR_H
#define HVMEMPTR_H

#include "hstructures.h"

/*! \brief Create a pointer to virtual BRAM memory.
 *  
 *  @param entry List element maintained by the hvmem-module. The element's data field contains address to the physical BRAM-memory.
 *  @param size The size (in bytes) of the data the pointer points to.
 *  @return Virtual pointer pointing to nothing. NULL on failure.
 */
struct hvmemptr* hvmemptr_create(struct hlelement* entry, int size);

/*! \brief Point the virtual pointer to the given BRAM-entry.
 *  
 *  @param pointer Virtual pointer.
 *  @param entry List element maintained by the hvmem-module. The element's data field contains address to the physical BRAM-memory.
 *  @param size The size (in bytes) of the data the pointer points to.
 *  @return 0 on success. Negative on failure.
 */
int hvmemptr_set(struct hvmemptr* pointer, struct hlelement* entry, int size);

/*! \brief Get the list element pointing to physical address in BRAM.
 *  
 *  @param pointer Virtual pointer.
 *  @return List element maintained by the hvmem-module. The element's data field contains address to the physical BRAM-memory. NULL on failure.
 */
struct hlelement* hvmemptr_get_bramentry(struct hvmemptr* pointer);

/*! \brief Get the size of the data pointed to by pointer.
 *  
 *  @param pointer Virtual pointer.
 *  @param size The size (in bytes) of the data the pointer points to. Negative on failure.
 */
int hvmemptr_get_size(struct hvmemptr* pointer);


#endif

