#include <stdlib.h>
#include <check.h>
#include <string.h>

#include <hicap.h>
#include "test_hwos.h"

const char* icapdevice = "/var/icap";
int devicehandle = -1;

static void setup (void);
static void teardown (void);
static Suite* hicap_suite (void);


static void setup (void)
{
        devicehandle = hicap_open(icapdevice, "XC4VFX12");
        // Write to row 0, CLB column 21 and with no frame offset. Write 66 frames.
        hicap_write(devicehandle, "onedff_ise_part.bit", 0, 21, 0, 66);
}

static void teardown (void)
{
        hicap_close(devicehandle);
}

START_TEST (test_icap_write)
{
        fail_unless (devicehandle != -1, "Could not open device.");
}
END_TEST


static Suite* hicap_suite (void)
{
        Suite *s = suite_create ("hicap");

        /* Core test case */
        TCase *tc_core = tcase_create ("Core");
        tcase_add_checked_fixture (tc_core, setup, teardown);
        tcase_add_test (tc_core, test_icap_write);
        suite_add_tcase (s, tc_core);

        return s;
}

int test_hicap (void)
{
        int number_failed;
        Suite *s = hicap_suite ();
        SRunner *sr = srunner_create (s);
        srunner_set_log (sr, TEST_LOG_HICAP);
        srunner_run_all (sr, CK_VERBOSE);

        number_failed = srunner_ntests_failed (sr);
        srunner_free (sr);
        return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}

