#include <stdlib.h>
#include <check.h>
#include <string.h>

//! Process interface.
#include <hprocess.h>
#include <hplacer.h>
#include "test_hwos.h"

static struct hprocess* process0 = NULL;

static void setup (void);
static void teardown (void);
static Suite* hplacer_suite (void);


static void setup (void)
{
        process0 = hprocess_create(0);
        hprocess_set_bitfilename(process0, "test_process0.bit");
}

static void teardown (void)
{
        hprocess_remove(process0);
}

START_TEST (test_place_process)
{
        fail_unless (hprocess_get_pid (process0) == 1, "PID not set correctly on creation.");
        fail_unless (hplacer_load_process(process0) == 0, "Could not load process into FPGA.");
}
END_TEST

static Suite* hplacer_suite (void)
{
        Suite *s = suite_create ("hplacer");

        /* Core test case */
        TCase *tc_core = tcase_create ("Core");
        tcase_add_checked_fixture (tc_core, setup, teardown);
        tcase_add_test (tc_core, test_place_process);
        suite_add_tcase (s, tc_core);

        return s;
}

int test_hplacer (void)
{
        int number_failed;
        Suite *s = hplacer_suite ();
        SRunner *sr = srunner_create (s);
        srunner_set_log (sr, TEST_LOG_HPLACER);
        srunner_run_all (sr, CK_VERBOSE);

        number_failed = srunner_ntests_failed (sr);
        srunner_free (sr);
        return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}

