#include <stdlib.h>
#include <check.h>
#include <string.h>

//! Process interface.
#include <hprocess.h>
#include "test_hwos.h"


static void setup (void);
static void teardown (void);
static Suite* hprocess_suite (void);


static struct hprocess* process0;
static struct hprocess* process1;
static struct hprocess* process2;
static struct hprocess* process3;

static void setup (void)
{
        process0 = hprocess_create(11);
        process1 = hprocess_create(1);
        process2 = hprocess_create(3);
        process3 = hprocess_create(6);
        hprocess_set_bitfilename(process0, "This is a test...");
}

static void teardown (void)
{
        hprocess_remove(process0);
        hprocess_remove(process1);
        hprocess_remove(process2);
        hprocess_remove(process3);
}

START_TEST (test_process_create)
{
        fail_unless (hprocess_get_pid (process0) == 1, "PID not set correctly on creation!");
        fail_unless (hprocess_get_pid (process1) == 2, "PID not set correctly on creation!");
        fail_unless (hprocess_get_pid (process2) == 3, "PID not set correctly on creation!");
        fail_unless (hprocess_get_pid (process3) == 4, "PID not set correctly on creation!");
        fail_unless (hprocess_get_nice (process0) == 11, "Nice not set correctly on creation!");
        fail_unless (hprocess_get_nice (process1) == 1, "Nice not set correctly on creation!");
        fail_unless (hprocess_get_nice (process2) == 3, "Nice not set correctly on creation!");
        fail_unless (hprocess_get_nice (process3) == 6, "Nice not set correctly on creation!");

        fail_unless (strcmp(hprocess_get_bitfilename(process0), "This is a test...") == 0, "Bitfilename is not set correct.");
        fail_unless (strlen(hprocess_get_bitfilename(process0)) == 17, "Bitfilename has wrong length.");
        fail_unless (hprocess_get_bitfilename(process0)[17] == '\0', "Bitfilename is not null-terminated.");
        fail_unless (strcmp(hprocess_get_bitfilename(process0), "Not this string!") != 0, "Bitfilename is not set correct.");
        hprocess_set_bitfilename(process0, "Short str.");
        fail_unless (strlen(hprocess_get_bitfilename(process0)) == 10, "Bitfilename has wrong length.");
        fail_unless (hprocess_get_bitfilename(process0)[10] == '\0', "Bitfilename is not null-terminated.");
        fail_unless (strcmp(hprocess_get_bitfilename(process0), "Short str.") == 0, "Bitfilename is not set correct.");
}
END_TEST

static Suite* hprocess_suite (void)
{
        Suite *s = suite_create ("hprocess");

        /* Core test case */
        TCase *tc_core = tcase_create ("Core");
        tcase_add_checked_fixture (tc_core, setup, teardown);
        tcase_add_test (tc_core, test_process_create);
        suite_add_tcase (s, tc_core);

        return s;
}

int test_hprocess (void)
{
        int number_failed;
        Suite *s = hprocess_suite ();
        SRunner *sr = srunner_create (s);
        srunner_set_log (sr, TEST_LOG_HPROCESS);
        srunner_run_all (sr, CK_VERBOSE);

        number_failed = srunner_ntests_failed (sr);
        srunner_free (sr);
        return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}

