#include <stdlib.h>
#include <stdio.h>
#include <check.h>
#include <errno.h>
#include <string.h>

#include <hmsg.h>
#include <hmqueue.h>
#include <hlog.h>
#include <integer_functions.h>
#include "test_hwos.h"


static void setup (void);
static void teardown (void);
static void setup_rmqueue (void);
static Suite* hwos_daemon_suite (void);


void* msg_from_daemon = NULL;
void* msg_to_daemon = NULL;
//! Message queues for the daemon and this client.
struct hmqueue* msq_this = NULL;
struct hmqueue* msq_daemon = NULL;
//! Just some random key to identify our queue.
int msqkey_this = 4321;


static void setup (void)
{
        int msqkey_daemon = hmqueue_get_daemonkey();
        msq_this = hmqueue_add(msqkey_this, HMQ_CREATE);
        msq_daemon = hmqueue_add(msqkey_daemon, HMQ_CONNECT);
        msg_to_daemon = hmsg_create(HMT_CTRL);
        
        hmsg_set_sender(msg_to_daemon, msqkey_this);
}


static void teardown (void)
{
        //! Unregister the queue.
        hmsg_set_command(msg_to_daemon, HMC_UMQUE);
        hmsg_send(msg_to_daemon, msq_daemon);

        hmqueue_remove(msq_this); 
        hmsg_remove(msg_to_daemon);
        hmsg_remove(msg_from_daemon);
}


static void setup_rmqueue (void)
{
        setup();
        hmsg_set_command(msg_to_daemon, HMC_RMQUE);
        if (hmsg_send(msg_to_daemon, msq_daemon) < 0) {
                hlog_write(HLOG_ERROR, "Could not send message. errno=");
                hlog_write_integer(errno);
                hlog_write_text(".\n");
        }
        msg_from_daemon = hmsg_receive(msq_this);
}


START_TEST (test_setup)
{
        fail_unless (hmsg_get_type(msg_to_daemon) == HMT_CTRL, "Incorrect message type.");
        fail_unless (hmsg_get_sender(msg_to_daemon) == msqkey_this, "Sender key not set for message to daemon.");
        fail_unless (hmqueue_get_id(msq_this) > -1, "Message queue for this client not registered.");
        fail_unless (hmqueue_get_id(msq_daemon) > -1, "Message queue for daemon not registered.");
}
END_TEST


START_TEST (test_rmqueue)
{
        fail_unless (hmsg_get_command(msg_to_daemon) == HMC_RMQUE, "Incorrect message command = %d", hmsg_get_command(msg_to_daemon));
        fail_unless (hmsg_get_return(msg_from_daemon) == HMR_OK, "Incorrect return value from daemon. It was %d", hmsg_get_return(msg_from_daemon));
}
END_TEST


START_TEST (test_register_process)
{
        void* regproc_msg = NULL;
        void* rec_msg = NULL;
        regproc_msg = hmsg_create(HMT_REGPROC);
        hmsg_set_sender(regproc_msg, msqkey_this);
        hmsg_set_bitfilename(regproc_msg, "adder_part.bit");
        hmsg_send(regproc_msg, msq_daemon);
        rec_msg = hmsg_receive(msq_this);

        fail_unless (hmsg_get_return(rec_msg) == HMR_OK, "Incorrect return value from daemon. It was %d", hmsg_get_return(msg_from_daemon));
        fail_unless (strcmp(hmsg_get_bitfilename(regproc_msg), "adder_part.bit") == 0, "Bitfilename is not set correct. It was: %s", hmsg_get_bitfilename(regproc_msg));

        hmsg_remove(regproc_msg);
        hmsg_remove(rec_msg);
}
END_TEST



static Suite* hwos_daemon_suite (void)
{
        Suite *s = suite_create ("hwos_daemon");

        /* Core test case */
        TCase *tc_core = tcase_create ("Core");
        tcase_add_checked_fixture (tc_core, setup, teardown);
        tcase_add_test (tc_core, test_setup);
        suite_add_tcase (s, tc_core);

        TCase *tc_msgpass = tcase_create ("Register message queue");
        tcase_add_checked_fixture (tc_msgpass, setup_rmqueue, teardown);
        tcase_add_test (tc_msgpass, test_setup);
        tcase_add_test (tc_msgpass, test_rmqueue);
        suite_add_tcase (s, tc_msgpass);

        TCase *tc_process = tcase_create ("Process");
        tcase_add_checked_fixture (tc_process, setup_rmqueue, teardown);
        tcase_add_test (tc_process, test_register_process);
        suite_add_tcase (s, tc_process);

        return s;
}


int test_hwos_daemon (void)
{
        int number_failed;
        Suite *s = hwos_daemon_suite ();
        SRunner *sr = srunner_create (s);
        srunner_set_log (sr, TEST_LOG_HWOS_DAEMON);
        srunner_run_all (sr, CK_VERBOSE);

        number_failed = srunner_ntests_failed (sr);
        srunner_free (sr);
        return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}

