-------------------------------------------------------------------------------
--
-- Title       : shift_register_counter
-- Design      : sequential multiplier
-- Author      : Vegard
-- Company     : NTNU
-- Description : Counter implemented as a hard register for save/load state.
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_unsigned.all;

entity counter_reg is
	generic(
		--register size
		C_SIZE	: natural :=8 
	);

	port(
		clk			: in  std_logic;  
		reset		: in  std_logic;
		count		: in  std_logic;
		write		: in  std_logic;
		data_i		: in  std_logic_vector(C_SIZE-1 downto 0);
		data_o		: out std_logic_vector(C_SIZE-1 downto 0)
	 );
end counter_reg;


architecture RTL of counter_reg is

signal memory : std_logic_vector(C_SIZE-1 downto 0);

begin
	
	data_o <= memory;

	
	CNT_PROC : process(clk) is
	begin
		if clk'event and clk ='1' then
			if reset ='1' then
				memory <= (others =>'0');
			elsif write ='1' then 
				memory <= data_i;
			elsif count ='1' then
				 memory <= memory+1;
			else
				memory <= memory;
			end if;
		end if;
	end process CNT_PROC;
end RTL;
