-------------------------------------------------------------------------------
--
-- Title       : instr_decoder
-- Design      : cpu_interface
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity instr_decoder is
	generic(
		INSTR_SIZE	: natural := 16
	);
	port(
		opcode 			: in std_logic_vector(1 downto 0);
		operand			: in std_logic_vector(INSTR_SIZE-3 downto 0);
		read_ack 		: in std_logic;	
		write_ack		: in std_logic;
		shift_complete	: in std_logic;
		hw_complete		: in std_logic; 
		cs				: in std_logic_vector(0 to 0);
		address			: out std_logic_vector(7 downto 0);
		rnw				: out std_logic;
		register_write	: out std_logic;
		hw_en			: out std_logic;
		hw_we			: out std_logic;
		count			: out std_logic_vector(1 downto 0);
		shift_mask		: out std_logic;
		zero_mask	    : out std_logic;
		hw_run     		: out std_logic;
		instr_complete  : out std_logic;
		shift_o			: out std_logic;
		halt			: out std_logic
		
	);
end instr_decoder;


architecture instr_decoder of instr_decoder is

constant READ 	: std_logic_vector :="00";
constant WRITE 	: std_logic_vector :="01";
constant SHIFT 	: std_logic_vector :="10"; 
constant RUN 	: std_logic_vector :="11";


begin  			 
	
	register_write <= read_ack;
	
	HW_EN_PROC : process(opcode,cs) is
	begin
		if (opcode = WRITE or opcode = READ) and cs(0)='0' then
			hw_en <='1';
		else
			hw_en <='0';
		end if;
	end process HW_EN_PROC;
	
	HW_WE_PROC : process(opcode) is
	begin
		if opcode = WRITE and cs(0)='0' then
			hw_we <='1';
		else
			hw_we <='0';
		end if;
	end process HW_WE_PROC;
	
	
	ADDRESS_PROC : process(opcode,operand) is
	begin
		if opcode = READ or opcode= WRITE then
			address <= operand(7 downto 0);
		elsif opcode  =SHIFT then
			address(1 downto 0 ) <= operand(3 downto 2);
			address(7 downto 2) <= (others =>'0');
		else
			address <= (others => '0');
		end if;
	end process ADDRESS_PROC;  
	
	COUNT_PROC : process(opcode,operand) is
	begin
		if opcode = SHIFT then
			count <= operand(1 downto 0);
		else
			count <= (others =>'0');
		end if;
	end process COUNT_PROC;
	
	SHIFT_MASK_PROC : process(opcode,operand) is
	begin
		if opcode =SHIFT then
			shift_mask <= operand(5);
		else
			shift_mask <='0';
		end if;
		
	end process SHIFT_MASK_PROC;
	
	ZERO_MASK_PROC : process(opcode,operand) is
	begin
		if opcode = SHIFT then
			zero_mask <= operand(4);
		else
			zero_mask <='0';
		end if;
	end process ZERO_MASK_PROC;
	
	INSTR_COMPLETE_PROC : process(opcode,operand,read_ack,write_ack,shift_complete,hw_complete) is
	begin
		case opcode is
			when READ =>
				 instr_complete <= read_ack;
			when WRITE =>
				 instr_complete <= write_ack;
			when SHIFT =>
				 instr_complete <= shift_complete; 
			when RUN =>							  
				instr_complete <= hw_complete;
			when others =>
				 instr_complete <='1';		
		end case;
		
	end process INSTR_COMPLETE_PROC;
	
	HW_RUN_PROC : process(hw_complete,opcode,operand) is
	begin
		if opcode = RUN and hw_complete ='0' then
			hw_run <='1';
		else
			hw_run <='0';
		end if;
	end process HW_RUN_PROC;
	
	HALT_PROC : process(opcode,operand) is
	begin
		if(opcode = RUN and operand(INSTR_SIZE-3) ='1') then
			halt <='1';
		else
			halt <='0';
		end if;	
	end process HALT_PROC;
	
	SHIFT_PROC : process(opcode) is
	begin
		if opcode = SHIFT then
			shift_o <='1';
		else
			shift_o <='0';
		end if;
	end process SHIFT_PROC;
	
	
end instr_decoder;
