-------------------------------------------------------------------------------
--
-- Title       : instr_controller
-- Design      : cpu_interface
-- Author      : Vegard
-- Company     : NTNU
-- Description : 
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
entity instr_fetcher is						  
	generic(			
		INSTR_ADDR_BITS : natural :=5
	);
	port(
		clk				: in std_logic;
		reset			: in std_logic;
		read_ack 		: in std_logic;	
		instr_complete	: in std_logic;
		halt			: in std_logic;
		instr_depth		: in std_logic_vector(INSTR_ADDR_BITS-1 downto 0);
		execute			: out std_logic;
		if_en			: out std_logic;
		instr_write		: out std_logic;
		instr_shift		: out std_logic;
		instr_addr_o	: out std_logic_vector(INSTR_ADDR_BITS-1 downto 0)
	);

end instr_fetcher;


architecture RTL of instr_fetcher is

type state is (INF,EX,WAITING,SHIFTING);
signal cstate,nstate : STATE;	


-- instruction fetch signals
--signal instr_addr   : std_logic_vector(3 downto 0);  -- instruction address counter  (modulo 8)
signal instr_modulo : std_logic;

signal instr_addr	:  std_logic_vector(INSTR_ADDR_BITS-1 downto 0);

begin	   
	
	instr_addr_o <= instr_addr;
	
	EXECUTE_PROC : process(cstate) is
	begin
		if cstate = EX then
			execute <='1';
		else
			execute <='0';
		end if;
	
	end process EXECUTE_PROC;
	
	INSTR_ADDR_PROC : process(reset,clk) is
	begin
		if clk'event and clk='1' then			
			if reset ='1' then
				instr_addr <= (others =>'0');
			elsif halt='1' then
				instr_addr <= instr_addr;
			elsif cstate = EX and nstate =WAITING then 
				if(instr_addr = instr_depth) then
					instr_addr <=(others =>'0');
				else
					instr_addr <= instr_addr +1;
				end if;
			end if;
		end if;
	end process INSTR_ADDR_PROC;
	
	INSTR_COUNT_PROC : process(reset,clk) is
	begin  
		if clk'event and clk='1' then
			if reset = '1' then 
				instr_modulo <= '0';
			elsif cstate = SHIFTING then
				instr_modulo <= '1';  
			elsif cstate = WAITING or cstate = INF then
				instr_modulo <= '0';
			end if;
		end if;
		
	end process INSTR_COUNT_PROC;
	
	INSTR_WRITE_PROC: process(cstate,read_ack) is
	begin
		if (cstate = INF and read_ack ='1') then
			instr_write <='1';
		else
			instr_write <='0';
		end if;
	end process INSTR_WRITE_PROC;
	
	INSTR_SHIFT_PROC : process(cstate,halt) is
	begin
		if cstate = SHIFTING and halt='0' then 
			instr_shift <='1';
		else
			instr_shift <='0';
		end if;
	end process INSTR_SHIFT_PROC;
	
	READ_REQ_PROC : process(cstate) is
	begin
		if (cstate = WAITING or cstate = INF)  then
			if_en <= '1';
		else
			if_en <='0';
		end if;
	end process READ_REQ_PROC;
	
	NSTATE_PROC : process(cstate,read_ack,instr_complete,instr_modulo,halt)is
	begin
		case cstate is 
			when INF =>	 
				--NOOP, force state INF
				if halt ='1' then
					nstate <= INF;
				elsif read_ack ='1' then 
					nstate <= EX;
				else
					nstate <= INF;
				end if;
			when EX	=>				
				if halt ='1' then
					nstate <= INF;
				elsif instr_complete ='1' then
					if instr_modulo ='0' then
						nstate <= SHIFTING;
					else		
						nstate <= WAITING;
					end if;
				else
					nstate <= EX;
				end if;	
			when SHIFTING =>
				nstate <= EX;
			when WAITING =>
				   nstate <= INF;
			end case;
	end process NSTATE_PROC;
	
	CSTATE_PROC : process(reset,clk) is
	begin
		if clk'event and clk='1' then
			if reset='1' then
				cstate <= INF;
			else
				cstate <= nstate;
			end if;
		end if;
	end process CSTATE_PROC;
						  
end RTL;
