-------------------------------------------------------------------------------
--
-- Title       : register_macro_s
-- Design      : ps_registers
-- Author      : Vegard	Endresen
-- Company     : NTNU
-- Description : Register macro, parallel input generic, serial and paralell shift
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_macro_s is	  
	generic(
		C_SIZE : natural :=8;   	 -- size of each register
		C_NUM_REG : natural := 4;	 --	number of registers	
		C_INPUT_SIZE : natural :=32  -- input width 
	);
	port(	   
		clk		: in  std_logic;
		reset   : in  std_logic;
		data_i	: in  std_logic_vector(C_INPUT_SIZE-1 downto 0); 
		write	: in  std_logic;
		shift_i	: in  std_logic_vector(C_SIZE-1 downto 0);
		shift_p	: in  std_logic; -- shift parallel
		shift_s	: in  std_logic; -- shift serial
		data_o	: out std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);
		shift_o	: out std_logic_vector(C_SIZE-1 downto 0)
	);
end register_macro_s;


architecture RTL of register_macro_s is

	
signal reg_inputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  
signal reg_outputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  
signal write_i : std_logic;

begin 		 
	
	write_i <= shift_p or write;
	
	OUT_PROC : process(reg_outputs)
	begin				
		shift_o <= reg_outputs(C_SIZE-1 downto 0);
		data_o<= reg_outputs;
		
	end process OUT_PROC;
	
	MUX_PROC : process(shift_p,shift_s,write,data_i,reg_outputs) is
	begin
		-- shift operation selected
		if shift_p ='1' or shift_s='1' then 
			reg_inputs(C_SIZE*C_NUM_REG -1 downto C_SIZE*(C_NUM_REG-1)) <= shift_i;
			for i in C_NUM_REG-1 downto 1 loop
				reg_inputs(C_SIZE*i -1 downto C_SIZE*(i-1)) <= reg_outputs(C_SIZE*(i+1) -1 downto C_SIZE*(i));
			end loop;
		-- parallel write operation selceted
		else
			for i in C_SIZE*C_NUM_REG-1 downto C_SIZE*C_NUM_REG-C_INPUT_SIZE loop
				reg_inputs(i) <= data_i(i-(C_SIZE*C_NUM_REG -C_INPUT_SIZE));
			end loop; 
			for i in C_SIZE*C_NUM_REG-C_INPUT_SIZE-1 downto 0 loop
				reg_inputs(i) <= reg_outputs(i);
			end loop; 
			
		end if;
	end process MUX_PROC;
	REG_GEN: for i in C_NUM_REG downto 1 generate
		REG : entity register_block_s
			port map(
				clk => clk,
				reset => reset,
				write => write_i,
				shift  => shift_s,	 
				data_i => reg_inputs(i*8-1 downto (i-1)*8),
				data_o => reg_outputs(i*8-1 downto (i-1)*8)	
				);
	end generate;

end RTL;
