-------------------------------------------------------------------------------
--
-- Title       : register_macro12
-- Design      : parallel_blocks
-- Author      : Vegard
-- Company     : NTNU
-- Description : Register macro, shift input only.
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_macro_so is 
	generic(
		C_SIZE : natural :=8;
		C_NUM_REG : natural := 4
	);
	port(	   
		clk		: in  std_logic;
		reset	: in  std_logic;
		shift_i : in  std_logic_vector(C_SIZE-1 downto 0);
		shift_s : in  std_logic;
		shift_p	: in  std_logic;
		shift_o	: out std_logic_vector(C_SIZE-1 downto 0)
	);
	
end register_macro_so;

architecture RTL of register_macro_so is



signal reg_inputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  
signal reg_outputs 	: std_logic_vector(C_SIZE*C_NUM_REG -1 downto 0);  


begin 		 
	
	
	
	OUT_PROC : process(reg_outputs,shift_i)
	begin				
		shift_o <= reg_outputs(C_SIZE-1 downto 0);
		reg_inputs(C_SIZE*C_NUM_REG -1 downto C_SIZE*(C_NUM_REG -1)) <= shift_i;
		for i in C_NUM_REG-1 downto 1 loop
			reg_inputs(C_SIZE*i -1 downto C_SIZE*(i-1)) <= reg_outputs(C_SIZE*(i+1) -1 downto C_SIZE*(i));
		end loop;
		
		
	end process OUT_PROC;
	
	
	REG_GEN: for i in C_NUM_REG downto 1 generate
		REG : entity register_block_s
			port map(
				clk => clk,
				reset => reset,
				data_i => reg_inputs(i*8-1 downto (i-1)*8),
				shift  => shift_s,
				write => shift_p,
				data_o => reg_outputs(i*8-1 downto (i-1)*8)	
				);
	end generate;
end RTL;
