-------------------------------------------------------------------------------
--
-- Title       : shift_alt
-- Design      : sequential multiplier
-- Author      : Vegard
-- Company     : NTNU
--
-------------------------------------------------------------------------------
--
-- File        : shift_alt.vhd
-- Generated   : Wed Mar 17 08:47:57 2010
-- From        : interface description file
-- By          : Itf2Vhdl ver. 1.20
--
-------------------------------------------------------------------------------
--
-- Description : 
--
-------------------------------------------------------------------------------

--{{ Section below this comment is automatically maintained
--   and may be overwritten
--{entity {shift_alt} architecture {shift_alt}}

library IEEE;
use IEEE.STD_LOGIC_1164.all;	
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity shift_alt is
	generic(C_COUNT_SIZE : natural :=5);
	port(
		clk				: in  std_logic;
		shift_i			: in  std_logic;
		count			: in  std_logic_vector(C_COUNT_SIZE-1 downto 0);
		shift_o			: out std_logic;
		shift_complete	: out std_logic
		);
end shift_alt;


architecture RTL of shift_alt is
signal cnt 	: std_logic_vector(C_COUNT_SIZE-1 downto 0);
 	type state	is (IDLE,COUNTING);
	signal cstate, nstate : state;
	
begin
	
	NSTATE_PROC : process(cstate,shift_i,cnt,count) is
	begin
		case cstate is
			when IDLE =>  
			if(shift_i)='1' then
				nstate <= COUNTING;
			else
				nstate <= IDLE;
			end if;
			when COUNTING =>
				if(cnt = count) then
					nstate <= IDLE;
				else			  
					nstate <= COUNTING;
				end if;
			end case;	
	end process NSTATE_PROC;
	
	CSTATE_PROC : process(clk)is
	begin					   
		if clk'event and clk ='1' then
			cstate <= nstate;
		end if;
	end process CSTATE_PROC;
	
	SHIFT_PROC : process(cstate) is
	begin
		if cstate = IDLE then
			shift_o <='0';
		else
			shift_o <='1';	
		end if;
	end process SHIFT_PROC;
	
	COUNT_PROC : process(clk) is
	begin
		if(clk'event and clk ='1') then
			if cstate = IDLE then
				cnt <=(others => '0');
			else
				cnt <= cnt+1;
			end if;
		end if;
	end process COUNT_PROC;
	

	COMPLETION_PROC: process(count,nstate) is
	begin
			if nstate = IDLE then
				shift_complete <='1';
			else
				shift_complete <='0';
			end if;
	
	end process COMPLETION_PROC;

end RTL;
