-- Top-level entitty for reconfigurable test module:
-- One flip-flop.
--
-- Sindre Hansen (2011). Module originally from my project.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.vcomponents.all; 

entity reconf is
		port (
	 clk_in	  : in   std_logic;
	 in_16    : in   std_logic_vector(0 to 15);
	 out_16   : out  std_logic_vector(0 to 15)
	);
end entity reconf;


architecture IMP of reconf is

signal bm_l2r_in : std_logic_vector(0 to 15);
signal bm_l2r_out : std_logic_vector(0 to 15);

signal bm_r2l_in : std_logic_vector(0 to 15);
signal bm_r2l_out : std_logic_vector(0 to 15);

signal clk_buffer_out : std_logic;
signal clk_buffer_ce : std_logic;

component busmacro_xc4v_l2r_async_narrow is
    	port (
    		input0 : in std_logic;
    		input1 : in std_logic;
			input2 : in std_logic;
    		input3 : in std_logic;
			input4 : in std_logic;
    		input5 : in std_logic;
			input6 : in std_logic;
    		input7 : in std_logic;
    		output0 : out std_logic;
    		output1 : out std_logic;
			output2 : out std_logic;
    		output3 : out std_logic;
			output4 : out std_logic;
    		output5 : out std_logic;
			output6 : out std_logic;
    		output7 : out std_logic
    	);
	end component;


component busmacro_xc4v_r2l_async_narrow is
    	port (
    		input0 : in std_logic;
    		input1 : in std_logic;
			input2 : in std_logic;
    		input3 : in std_logic;
			input4 : in std_logic;
    		input5 : in std_logic;
			input6 : in std_logic;
    		input7 : in std_logic;
    		output0 : out std_logic;
    		output1 : out std_logic;
			output2 : out std_logic;
    		output3 : out std_logic;
			output4 : out std_logic;
    		output5 : out std_logic;
			output6 : out std_logic;
    		output7 : out std_logic
    	);
	end component;



begin

	-- Connect the outputs & inputs to the bus macro outputs & inputs 
	out_16 <= bm_r2l_out;
	bm_l2r_in <= in_16;

	R : entity reconfigurable_module
    	port map
    	(
		clk_in => clk_buffer_out,
		in_16 => bm_l2r_out,
		out_16 => bm_r2l_in
	);

	-- Connect one of the outputs from the HWSW-register to clk_buffer_ce.
	-- This means the clock buffer's CE-input can be controlled by a Linux-app.
	clk_buffer_ce <= bm_l2r_in(2);

	-- Instantiate global clock buffer explicitely to enable/disable clock
	BUFGCE_inst : BUFGCE
	port map (
						 O => clk_buffer_out,		-- Clock buffer output (connected to reconf. module)
						 CE => clk_buffer_ce,		-- Clock enable input
						 I => clk_in						-- Clock buffer input
					 );
	

	bm_l2r_0 : component busmacro_xc4v_l2r_async_narrow
		port map(
				--
			input0 =>bm_l2r_in(0),
			input1 =>bm_l2r_in(1),
			input2 =>bm_l2r_in(2),
			input3 =>bm_l2r_in(3),
			input4 =>bm_l2r_in(4),
			input5 =>bm_l2r_in(5),
			input6 =>bm_l2r_in(6),
			input7 =>bm_l2r_in(7),
			output0 =>bm_l2r_out(0),
			output1 =>bm_l2r_out(1),
			output2 =>bm_l2r_out(2),
			output3 =>bm_l2r_out(3),
			output4 =>bm_l2r_out(4),
			output5 =>bm_l2r_out(5),
			output6 =>bm_l2r_out(6),
			output7 =>bm_l2r_out(7)
			
		);
		
		bm_l2r_1 : component busmacro_xc4v_l2r_async_narrow
		port map(
				
			input0 =>bm_l2r_in(8),
			input1 =>bm_l2r_in(9),
			input2 =>bm_l2r_in(10),
			input3 =>bm_l2r_in(11),
			input4 =>bm_l2r_in(12),
			input5 =>bm_l2r_in(13),
			input6 =>bm_l2r_in(14),
			input7 =>bm_l2r_in(15),
			output0 =>bm_l2r_out(8),
			output1 =>bm_l2r_out(9),
			output2 =>bm_l2r_out(10),
			output3 =>bm_l2r_out(11),
			output4 =>bm_l2r_out(12),
			output5 =>bm_l2r_out(13),
			output6 =>bm_l2r_out(14),
			output7 =>bm_l2r_out(15)
			
		);
		
		bm_r2l_0 : component busmacro_xc4v_r2l_async_narrow
		port map(
				
			input0 =>bm_r2l_in(0),
			input1 =>bm_r2l_in(1),
			input2 =>bm_r2l_in(2),
			input3 =>bm_r2l_in(3),
			input4 =>bm_r2l_in(4),
			input5 =>bm_r2l_in(5),
			input6 =>bm_r2l_in(6),
			input7 =>bm_r2l_in(7),
			output0 =>bm_r2l_out(0),
			output1 =>bm_r2l_out(1),
			output2 =>bm_r2l_out(2),
			output3 =>bm_r2l_out(3),
			output4 =>bm_r2l_out(4),
			output5 =>bm_r2l_out(5),
			output6 =>bm_r2l_out(6),
			output7 =>bm_r2l_out(7)
			
		);
		
	
	
		bm_r2l_1 : component busmacro_xc4v_r2l_async_narrow
		port map(
				
			input0 =>bm_r2l_in(8),
			input1 =>bm_r2l_in(9),
			input2 =>bm_r2l_in(10),
			input3 =>bm_r2l_in(11),
			input4 =>bm_r2l_in(12),
			input5 =>bm_r2l_in(13),
			input6 =>bm_r2l_in(14),
			input7 =>bm_r2l_in(15),
			output0 =>bm_r2l_out(8),
			output1 =>bm_r2l_out(9),
			output2 =>bm_r2l_out(10),
			output3 =>bm_r2l_out(11),
			output4 =>bm_r2l_out(12),
			output5 =>bm_r2l_out(13),
			output6 =>bm_r2l_out(14),
			output7 =>bm_r2l_out(15)
			
		);

end IMP;
