-- Reconfigurable test module:
-- Three flip-flops.
--
-- Sindre Hansen (2011). Module originally from my project.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity reconfigurable_module is
	port (
	 clk_in	  : in   std_logic;
	 in_16    : in   std_logic_vector(0 to 15);
	 out_16   : out  std_logic_vector(0 to 15)
	);
end entity reconfigurable_module;

architecture arch of reconfigurable_module is
	signal rst_in : std_logic;		-- Reset DFF
	signal reg0_in : std_logic;   -- DFF input
	signal reg0_out : std_logic;  -- DFF output
	signal reg1_in : std_logic;   -- DFF input
	signal reg1_out : std_logic;  -- DFF output
	signal reg2_in : std_logic;   -- DFF input
	signal reg2_out : std_logic;  -- DFF output
begin
	
	-- D-flip-flop
	-- in_16
	--   bit 0-2: d0-d2
	--   bit 3: rst
	--   bit 4-15 : not used (but connected to output to avoid error messages)
	-- out_16
	--   bit 0-2: q0-q2
	--   bit 3-14: not used (but connected to input bits 4-15 anyway)
	--   bit 15: not used (set to '0')

	reg0_in <= in_16(0);
	reg1_in <= in_16(1);
	reg2_in <= in_16(2);
	rst_in <= in_16(3);
	out_16(0) <= reg0_out;
	out_16(1) <= reg1_out;
	out_16(2) <= reg2_out;
	out_16(3 to 14) <= in_16(4 to 15);
	out_16(15) <= '0';
	
	OUT_PROC : process(clk_in, rst_in)
	begin
		if (rst_in='1') then
			reg0_out <= '0';
			reg1_out <= '0';
			reg2_out <= '0';
		elsif (clk_in'EVENT and clk_in='1') then
			reg0_out <= reg0_in;
			reg1_out <= reg1_in;
			reg2_out <= reg2_in;
		end if;
	end process OUT_PROC;
	
end arch;
