-- Reconf. module for design:
-- Two DFFs. One for initiliazing the clock signal to the module and one
-- for occupying the lowest clock wire.
--
-- Sindre Hansen (2011).


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity reconfigurable_module is
	port (
	 clk_in	  : in   std_logic;
	 in_16    : in   std_logic_vector(0 to 15);
	 out_16   : out  std_logic_vector(0 to 15)
	);
end entity reconfigurable_module;

architecture arch of reconfigurable_module is
begin
	
	-- D-flip-flop
	-- in_16
	--   bit 0: d
	--   bit 1: rst
	--   bit 2-15 : not used (but connected to output to avoid error messages)
	-- out_16
	--   bit 0: q
	--   bit 1-14: not used (but connected to input bits 2-15 anyway)
	--   bit 15: not used (set to '0')
	
	out_16(1 to 14) <= in_16(2 to 15);
	out_16(15) <= '0';
	
	OUT_PROC : process(clk_in, in_16(1))
	begin
		if (in_16(1)='1') then
			out_16(0) <= '0';
		elsif (clk_in'EVENT and clk_in='1') then
			out_16(0) <= in_16(0);
		end if;
	end process OUT_PROC;
	
end arch;
