-------------------------------------------------------------------------------
--
-- Title       : cla_adder
-- Design      : sequential_multiplier
-- Author      : Vegard
-- Company     : NTNU
-- Description : Parameterized CLA adder
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
library work;


entity cla_adder is
	generic( OPERAND_SIZE : natural :=4);	
	
	port(
		a_i		: in std_logic_vector(OPERAND_SIZE-1 downto 0);
		b_i		: in std_logic_vector(OPERAND_SIZE-1 downto 0);
		s_o		: out std_logic_vector(OPERAND_SIZE downto 0)
		);
end entity cla_adder;

architecture RTL of cla_adder is

signal carry_g	: std_logic_vector(OPERAND_SIZE-1 downto 0);
signal carry_p	: std_logic_vector(OPERAND_SIZE-1 downto 0);
signal carry_i	: std_logic_vector(OPERAND_SIZE downto 0);

begin
	CARRY_PROC: process(a_i,b_i,carry_g,carry_p,carry_i)
	begin
		carry_g 	<= a_i and b_i;
		carry_p 	<= a_i or  b_i;
		carry_i(0) 	<= '0';
		for i in 0 to OPERAND_SIZE-1 loop
			carry_i(i+1) <= carry_g(i) or (carry_p(i) and carry_i(i));
		end loop;
	end process CARRY_PROC;
	
	ADDERS_GEN: for i in 0 to OPERAND_SIZE-1 generate
		S0 : entity work.adder(add)
			port map(a_i => a_i(i) , b_i => b_i(i) , c_i=> carry_i(i) , s_o=> s_o(i));
	end generate;
	
	s_o(OPERAND_SIZE) <= carry_i(OPERAND_SIZE);
	
end RTL;