-------------------------------------------------------------------------------
--
-- Title       : register_block_s
-- Design      : ps_registers
-- Author      : Vegard Endresen
-- Company     : NTNU
-- Description : Paralell register with full I/O width which implements serial shift
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity register_block_s is
	generic(C_SIZE : natural :=8);
	port(	   
		clk		: in  std_logic;
		reset	: in  std_logic;
		shift	: in  std_logic;
		write	: in  std_logic;
		data_i  : in  std_logic_vector(C_SIZE-1 downto 0);
		data_o	: out std_logic_vector(C_SIZE-1 downto 0)
	);
end register_block_s ;


architecture RTL of register_block_s  is
signal memory : std_logic_vector(C_SIZE-1 downto 0);
begin
	
	data_o <= memory;
	
	write_proc : process(clk) is 
	begin  
		if clk'event and clk ='1' then
			if reset ='1' then
				memory <=(others =>'0');
			elsif write = '1' then 
				memory <= data_i;
			elsif shift ='1' then
				memory(C_SIZE-1) <= data_i(0);
				for i in C_SIZE-2 downto 0 loop
					memory(i) <= memory(i+1);		
				end loop;
			end if;
		end if;
		
	end process write_proc;
	
end RTL;
